package org.openmuc.jdlms;

import java.lang.annotation.Documented;
import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

/**
 * <p>
 * The {@link CosemClass} annotation tells jDLMS stack that the annotated class is a DLMS.
 * </p>
 * 
 * <pre>
 * &#64;{@link CosemClass}(id = 99, obis = "0.2.0.0.0.255", version = 2)
 * public class DemoClass {
 *      ...
 * }
 * </pre>
 * 
 * @see CosemMethod
 * @see CosemAttribute
 */
@Target(ElementType.TYPE)
@Retention(RetentionPolicy.RUNTIME)
@Documented
public @interface CosemClass {

    /**
     * The instance ID of the COSEM class.
     * 
     * <p>
     * NOTE: The instance ID is not allowed to use the instance ID of a COSEM system class.
     * </p>
     * 
     * @return the OBIS code string representation.
     */
    String obis();

    /**
     * The COSEM class ID. Value greater than zero.
     * 
     * @return the class ID.
     */
    int id();

    /**
     * The version of the COSEM class.
     * 
     * <p>
     * The version must be a positive number.
     * </p>
     * 
     * @return the version of the COSEM class.
     */
    int version() default 0;
}
