package org.openmuc.jdlms;

import org.junit.Test;
import org.openmuc.jdlms.datatypes.DataObject;

public class SelectivAccessServerTest {

    @Test(expected = IllegalPametrizationError.class)
    public void testSelectiveAccessExHandling() throws Exception {
        LogicalDevice logicalDevice = newDemoLD();
        logicalDevice.registerCosemClass(new DemoClass());
        DlmsServer.tcpServerBuilder(0).registerLogicalDevice(logicalDevice).build();
    }

    @Test(expected = IllegalPametrizationError.class)
    public void testExHandling2() throws Exception {
        LogicalDevice logicalDevice = newDemoLD();
        logicalDevice.registerCosemClass(new DemoClass2());
        DlmsServer.tcpServerBuilder(0).registerLogicalDevice(logicalDevice).build();
    }

    private LogicalDevice newDemoLD() {
        LogicalDevice logicalDevice = new LogicalDevice(1, "", "ISE", 1);
        return logicalDevice;
    }

    @CosemClass(id = 2, obis = "99.0.0.0.99.0")
    public class DemoClass {
        @CosemAttribute(id = 1)
        private DataObject d1;
    }

    @CosemClass(id = 2, obis = "99.0.0.0.99.0")
    public class DemoClass2 {
        @CosemAttribute(id = 2)
        private DataObject d1;
        @CosemAttribute(id = 2)
        private DataObject d2;
    }

    @Test(expected = IllegalPametrizationError.class)
    public void testExHandling3() throws Exception {
        LogicalDevice logicalDevice = newDemoLD();
        logicalDevice.registerCosemClass(new DemoClass3(), new DemoClass3());
        DlmsServer.tcpServerBuilder(0).registerLogicalDevice(logicalDevice).build();
    }

    @CosemClass(id = 2, obis = "99.0.0.0.99.0")
    public class DemoClass3 {
        @CosemAttribute(id = 2)
        private DataObject d1;
    }

    @Test(expected = IllegalPametrizationError.class)
    public void testExHandling4() throws Exception {
        LogicalDevice logicalDevice = newDemoLD();
        logicalDevice.registerCosemClass(new DemoClass4());
        DlmsServer.tcpServerBuilder(0).registerLogicalDevice(logicalDevice).build();
    }

    @CosemClass(id = 2, obis = "99.0.0.0.99.0")
    public class DemoClass4 {
        @CosemAttribute(id = 2, slector = 1)
        private DataObject d1;
    }

    @Test
    public void testExHandling5() throws Exception {
        LogicalDevice logicalDevice = newDemoLD();
        logicalDevice.registerCosemClass(new DemoClass5());
        DlmsServer.tcpServerBuilder(0).registerLogicalDevice(logicalDevice).build();
    }

    @CosemClass(id = 2, obis = "99.0.0.0.99.0")
    public class DemoClass5 {
        @CosemAttribute(id = 2)
        private DataObject d1;

        public DataObject getD1() {
            return d1;
        }

        public void setD1(DataObject d1) {
            this.d1 = d1;
        }
    }

    @Test
    public void testExHandling6() throws Exception {
        LogicalDevice logicalDevice = newDemoLD();
        logicalDevice.registerCosemClass(new DemoClass6());
        DlmsServer.tcpServerBuilder(0).registerLogicalDevice(logicalDevice).build();
    }

    @CosemClass(id = 2, obis = "99.0.0.0.99.0")
    public class DemoClass6 {
        @CosemAttribute(id = 2, slector = 1)
        private DataObject d1;

        public DataObject getD1(SelectiveAccessDescription selectiveAccessDescription) {
            return d1;
        }

        public void setD1(DataObject d1, SelectiveAccessDescription selectiveAccessDescription) {
            this.d1 = d1;
        }
    }

    @Test
    public void testExHandling7() throws Exception {
        LogicalDevice logicalDevice = newDemoLD();
        logicalDevice.registerCosemClass(new DemoClass7());
        DlmsServer.tcpServerBuilder(0).registerLogicalDevice(logicalDevice).build();
    }

    @CosemClass(id = 2, obis = "99.0.0.0.99.0")
    public class DemoClass7 {
        @CosemAttribute(id = 2)
        private DataObject d1;

        public DataObject getD1(Long id) {
            return d1;
        }

        public void setD1(DataObject d1) {
            this.d1 = d1;
        }
    }

    @Test(expected = IllegalPametrizationError.class)
    public void testExHandling8() throws Exception {
        LogicalDevice logicalDevice = newDemoLD();
        logicalDevice.registerCosemClass(new DemoClass8());
        DlmsServer.tcpServerBuilder(0).registerLogicalDevice(logicalDevice).build();
    }

    @CosemClass(id = 2, obis = "99.0.0.0.99.0")
    public class DemoClass8 {
        @CosemAttribute(id = 2)
        private DataObject d1;

        public DataObject getD1(SelectiveAccessDescription selectiveAccessDescription, Long id) {
            return d1;
        }

        public void setD1(DataObject d1) {
            this.d1 = d1;
        }
    }

    @Test
    public void testExHandling9() throws Exception {
        LogicalDevice logicalDevice = newDemoLD();
        logicalDevice.registerCosemClass(new DemoClass9());
        DlmsServer.tcpServerBuilder(0).registerLogicalDevice(logicalDevice).build();
    }

    @CosemClass(id = 2, obis = "99.0.0.0.99.0")
    public class DemoClass9 {
        @CosemAttribute(id = 2, slector = 2)
        private DataObject d1;

        public DataObject getD1(SelectiveAccessDescription selectiveAccessDescription, Long id) {
            return d1;
        }

        public void setD1(DataObject d1, SelectiveAccessDescription selectiveAccessDescription) {
            this.d1 = d1;
        }
    }
}
