/*
 * Copyright 2012-16 Fraunhofer ISE
 *
 * This file is part of jDLMS.
 * For more information visit http://www.openmuc.org
 *
 * jDLMS is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * jDLMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with jDLMS.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package org.openmuc.jdlms.internal.sessionlayer.hdlc;

import static org.junit.Assert.assertEquals;

import org.junit.Test;
import org.openmuc.jdlms.HexConverter;

public class HdlcAddressTest {

    /*
     * See EN 62056-46:2002 + A1:2007 p. 40 for further information
     */

    @Test
    public void testSymmetry1() throws Exception {
        HdlcAddress address = new HdlcAddress(10);

        byte[] encodedAddress = address.encode();
        HdlcAddress decodedAddress = HdlcAddress.decode(encodedAddress, encodedAddress.length);

        assertEquals(address.logicalDeviceAddress(), decodedAddress.logicalDeviceAddress());
        assertEquals(address, decodedAddress);
    }

    @Test
    public void testSymmetry2() throws Exception {
        HdlcAddress address = new HdlcAddress(10, 12);

        byte[] encodedAddress = address.encode();
        HdlcAddress decodedAddress = HdlcAddress.decode(encodedAddress, encodedAddress.length);

        assertEquals(address.logicalDeviceAddress(), decodedAddress.logicalDeviceAddress());
        assertEquals(address, decodedAddress);
    }

    @Test
    public void testDecode() throws Exception {

        byte logicalDeviceAddress = 30;
        byte[] bytes = { (byte) (logicalDeviceAddress << 1 | 1) };

        HdlcAddress decodedAddress = HdlcAddress.decode(bytes, bytes.length);

        assertEquals(logicalDeviceAddress, decodedAddress.logicalDeviceAddress());
    }

    @Test
    public void testDecodingFromString1Byte() throws Exception {
        byte[] data = HexConverter.fromShortHexString("C9");
        int length = data.length;
        HdlcAddress address = HdlcAddress.decode(data, length);

        assertEquals(new HdlcAddress(100), address);
    }

    @Test
    public void testDecodingFromString2Byte() throws Exception {
        byte[] data = HexConverter.fromShortHexString("0223");
        int length = data.length;
        HdlcAddress address = HdlcAddress.decode(data, length);

        assertEquals(new HdlcAddress(1, 17), address);
    }

    @Test
    public void testDecodingFromString4Byte() throws Exception {
        byte[] data = HexConverter.fromShortHexString("02220003");
        int length = data.length;
        HdlcAddress address = HdlcAddress.decode(data, length);

        assertEquals(new HdlcAddress(145, 1), address);
    }

}
