package org.openmuc.jdlms.internal.sessionlayer;

import java.io.IOException;
import java.nio.ByteBuffer;

import org.openmuc.jdlms.internal.connectionSettings.ServerSettings;
import org.openmuc.jdlms.internal.sessionlayer.wrapper.WrapperHeader;
import org.openmuc.jdlms.internal.transportlayer.StreamAccessor;

public class ServerWrapperLayer implements ServerSessionLayer {

    private final StreamAccessor streamAccessor;

    private int logicalDevice;
    private int clientId;

    private WrapperHeader.WrapperHeaderBuilder headerBuilder;

    private final ServerSettings settings;

    public ServerWrapperLayer(StreamAccessor streamAccessor, ServerSettings settings) {
        this.streamAccessor = streamAccessor;
        this.settings = settings;

        this.logicalDevice = -1;
        this.clientId = -1;
    }

    @Override
    public void send(byte[] data, int offset, int length) throws IOException {
        if (this.headerBuilder == null) {
            // TODO error!!! serios bug in API..
        }

        this.headerBuilder.length(length);

        byte[] wpdu = ByteBuffer.allocate(length + WrapperHeader.HEADER_LENGTH)
                .put(headerBuilder.build().encode())
                .put(data, offset, length)
                .array();

        this.streamAccessor.outpuStream().write(wpdu);
        this.streamAccessor.outpuStream().flush();
    }

    @Override
    public byte[] readNextMessage() throws IOException {
        streamAccessor.setTimeout(this.settings.inactivityTimeout);
        WrapperHeader header = WrapperHeader.decode(streamAccessor, this.settings.responseTimeout);

        if (this.logicalDevice == -1 || this.clientId == -1) {
            this.logicalDevice = header.destinationWPort();
            this.clientId = header.sourceWPort();
            this.headerBuilder = WrapperHeader.builder(this.logicalDevice, this.clientId);
        }
        validateHeader(header);

        byte[] data = new byte[header.payloadLength()];
        this.streamAccessor.inputStream().read(data);

        return data;
    }

    private void validateHeader(WrapperHeader header) {
        if (this.logicalDevice != header.destinationWPort() || this.clientId != header.sourceWPort()) {
            // TODO
        }
    }

    @Override
    public int getClientId() {
        return this.clientId;
    }

    @Override
    public int getLogicalDeviceId() {
        return this.logicalDevice;
    }

    @Override
    public void close() throws IOException {
        this.streamAccessor.close();
    }

}
